import os
from Products.CMFCore.DirectoryView import addDirectoryViews, registerDirectory, \
     createDirectoryView, manage_listAvailableDirectories
from Products.CMFCore.utils import getToolByName, minimalpath
from Globals import package_home
from OFS.ObjectManager import BadRequestException

from Products.MimetypesRegistry import GLOBALS, skins_dir
from Products.MimetypesRegistry.interfaces import IMimetypesRegistry
from Acquisition import aq_base
from StringIO import StringIO

def install(self):
    out = StringIO()
    id = 'mimetypes_registry'
    if hasattr(aq_base(self), id):
        mtr = getattr(self, id)
        if not IMimetypesRegistry.isImplementedBy(mtr) or \
          not getattr(aq_base(mtr), '_new_style_mtr', None) == 1:
            print >>out, 'Removing old mimetypes registry tool'
            self.manage_delObjects([id,])
    if not hasattr(self, id):
        addTool = self.manage_addProduct['MimetypesRegistry'].manage_addTool
        addTool('MimeTypes Registry')
        print >>out, 'Installing mimetypes registry tool'

    skinstool=getToolByName(self, 'portal_skins')

    fullProductSkinsPath = os.path.join(package_home(GLOBALS), skins_dir)
    productSkinsPath = minimalpath(fullProductSkinsPath)
    registered_directories = manage_listAvailableDirectories()
    if productSkinsPath not in registered_directories:
        registerDirectory(skins_dir, GLOBALS)
    try:
        addDirectoryViews(skinstool, skins_dir, GLOBALS)
    except BadRequestException, e:
        pass  # directory view has already been added

    files = os.listdir(fullProductSkinsPath)
    for productSkinName in files:
        if os.path.isdir(os.path.join(fullProductSkinsPath, productSkinName)) \
               and productSkinName != 'CVS':
            for skinName in skinstool.getSkinSelections():
                path = skinstool.getSkinPath(skinName)
                path = [i.strip() for i in  path.split(',')]
                try:
                    if productSkinName not in path:
                        path.insert(path.index('custom') +1, productSkinName)
                except ValueError:
                    if productSkinName not in path:
                        path.append(productSkinName)
                path = ','.join(path)
                skinstool.addSkinSelection(skinName, path)

    return out.getvalue()



from Products.MimetypesRegistry.MimeTypeItem import MimeTypeItem
from Products.MimetypesRegistry.common import MimeTypeException


map = {
    # '.extension' : 'mimetype',
    '.svg' : 'image/svg+xml', # scaleable vector graphics
    '.pjpg' : 'image/pjpeg', # scaleable vector graphics
    
}

def initialize(registry):
    #Find things that are not in the specially registered mimetypes
    #and add them using some default policy, none of these will impl
    #iclassifier
    for ext, mt in map.items():
        if ext[0] == '.':
            ext = ext[1:]
        
        if registry.lookupExtension(ext):
            continue

        try:
            mto =  registry.lookup(mt)
        except MimeTypeException:
            # malformed MIME type
            continue
        if mto:
            mto = mto[0]
            if not ext in mto.extensions:
                registry.register_extension(ext, mto)
                mto.extensions += (ext, )
            continue
        isBin = mt.split('/', 1)[0] != "text"
        registry.register(MimeTypeItem(mt, (mt,), (ext,), isBin))



import re
import glob
from unittest import TestSuite
from sys import modules
from os.path import join, abspath, dirname, basename

def normalize_html(s):
    s = re.sub(r"\s+", " ", s)
    s = re.sub(r"(?s)\s+<", "<", s)
    s = re.sub(r"(?s)>\s+", ">", s)
    s = re.sub(r"\r", "", s)
    return s

PREFIX = abspath(dirname(__file__))

def input_file_path(file):
    return join(PREFIX, 'input', file)

def output_file_path(file):
    return join(PREFIX, 'output', file)

def matching_inputs(pattern):
    return [basename(path) for path in glob.glob(join(PREFIX, "input", pattern))]

def load(dotted_name, globals=None):
    """ load a python module from it's name """
    mod = __import__(dotted_name, globals)
    components = dotted_name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod



import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.Archetypes.tests.common import *

from Products.MimetypesRegistry.encoding import guess_encoding


class TestGuessEncoding(ArchetypesTestCase):

    def testUTF8(self):
        e = guess_encoding('\xef\xbb\xbf any UTF-8 data')
        self.failUnlessEqual(e, 'UTF-8')
        e = guess_encoding(' any UTF-8 data \xef\xbb\xbf')
        self.failUnlessEqual(e, None)

    def testEmacs(self):
        e = guess_encoding('# -*- coding: UTF-8  -*-')
        self.failUnlessEqual(e, 'UTF-8')
        e = guess_encoding('''
        ### -*- coding: ISO-8859-1  -*-
        ''')
        self.failUnlessEqual(e, 'ISO-8859-1')
        e = guess_encoding('''

        ### -*- coding: ISO-8859-1  -*-
        ''')
        self.failUnlessEqual(e, None)

    def testVim(self):
        e = guess_encoding('# vim:fileencoding=UTF-8')
        self.failUnlessEqual(e, 'UTF-8')
        e = guess_encoding('''
        ### vim:fileencoding=ISO-8859-1
        ''')
        self.failUnlessEqual(e, 'ISO-8859-1')
        e = guess_encoding('''

        ### vim:fileencoding= ISO-8859-1
        ''')
        self.failUnlessEqual(e, None)

    def testXML(self):
        e = guess_encoding('<?xml?>')
        self.failUnlessEqual(e, 'UTF-8')
        e = guess_encoding('''<?xml version="1.0" encoding="ISO-8859-1" ?>
        ''')
        self.failUnlessEqual(e, 'ISO-8859-1')
        e = guess_encoding('''<?xml version="1.0" encoding="ISO-8859-1"?>
        ''')
        self.failUnlessEqual(e, 'ISO-8859-1')
        e = guess_encoding('''<?xml version="1.0" encoding="ISO-8859-1"?><truc encoding="UTF-8">
        </truc>
        ''')
        self.failUnlessEqual(e, 'ISO-8859-1')

    def testHTML(self):
        e = guess_encoding('''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
<title>ASPN : Python Cookbook : Auto-detect XML encoding</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
    <meta name="robots" content="all" />
    <meta name="description" content="ActiveState Open Source Programming tools for Perl Python XML xslt scripting with free trials. Quality development tools for programmers systems administrators database administrators network administrators and webmasters" />
    <meta name="keywords" content="ActiveState,Perl,xml,xslt,mozilla,Open Source,Python,Perl for Win32,resources,PerlScript,ActivePerl,Programming,Programmers,Integrated,Development,Environment,SOAP,Linux,Solaris,Web,development,tools,free,software,download,support,Perl Resource Kit,System Administration,Sys Admin,WinNT,SQL,Oracle,Email,XML,Linux,Programming,perl,NT,2000,windows,Unix,Software,Security,   Administration,systems,windows,database,database,consulting,support,Microsoft,developer,resource,code,tutorials,IDE,Integrated development environment,developer,resources,tcl,php" />

<link rel="stylesheet" href="/ASPN/aspn.css" />

</head>

<body bgcolor="#FFFFFF" leftmargin="0" topmargin="0" marginwidth="0" marginheight="0">
charset=utf-8
</body>
</html> ''')
        self.failUnlessEqual(e, 'iso-8859-1')


    def test_broken_percent(self):
        e = guess_encoding(
r"""<pre>
&lt;metal:block tal:define="dummy python:
request.RESPONSE.setHeader('Content-Type',
'text/html;;charset=%s' % charset)" /&gt;
&lt;metal:block tal:define="dummy
python:request.RESPONSE.setHeader('Content-Language', lang)"
/
&gt;
</pre>
"""
    )
        # unable to detect a valid encoding
        self.failUnlessEqual(e, None) 


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestGuessEncoding))
    return suite

if __name__ == '__main__':
    framework()



"""
magic.py

 Initial Author: Jason Petrone <jp@demonseed.net>

 Updated by Gabriel Wicke <groups@gabrielwicke.de>
    Thu Oct 16 23:00:03 CEST 2003
    with magic data from gnome-vfs-mime-magic

"""

import re
import struct
import string
from StringIO import StringIO
from zipfile import ZipFile
from zipfile import BadZipfile
from xml.dom import minidom

__version__ = '$Revision: 1.2 $'[11:-2]

magic = [

    [0L, 'string', '=', '%PDF-', 'application/pdf'],
    [0L, 'string', '=', '\177ELF', 'application/x-executable-binary'],
    [0L, 'string', '=', '\004%!', 'application/postscript'],
    [0L, 'string', '=', '\000\000\001\272', 'video/mpeg'],
    [0L, 'string', '=', '\000\000\001\263', 'video/mpeg'],
    [0L, 'string', '=', '\x47\x3f\xff\x10', 'video/mpeg'],
    [0L, 'string', '=', '\377\330\377', 'image/jpeg'],
    [0L, 'string', '=', '\xed\xab\xee\xdb', 'application/x-rpm'],
    [0L, 'string', '=', 'Rar!', 'application/x-rar'],
    [257L, 'string', '=', 'ustar\0', 'application/x-tar'],
    [257L, 'string', '=', 'ustar\040\040\0', 'application/x-gtar'],
    # the following detection of OOo is according to 
    # http://books.evc-cit.info/oobook/ch01.html
    # and some heuristics found in hexeditor. if theres a better way to detect,
    # we should replace the signatures below.
    # best would to just read and evaluate the manifest file of the zip, but
    # the magic tests are running on the first 8kB, so we cant unzip the 
    # manifest in files >8kB.
    [30L, 'string', '=', 'mimetypeapplication/vnd.sun.xml.writer', 
     'application/vnd.sun.xml.writer'],
    [30L, 'string', '=', 'mimetypeapplication/vnd.sun.xml.calc', 
     'application/vnd.sun.xml.calc'],
    [30L, 'string', '=', 'mimetypeapplication/vnd.sun.xml.draw', 
     'application/vnd.sun.xml.draw'],
    [30L, 'string', '=', 'mimetypeapplication/vnd.sun.xml.impress', 
     'application/vnd.sun.xml.impress'],
    [30L, 'string', '=', 'mimetypeapplication/vnd.sun.xml.chart', 
     'application/vnd.sun.xml.chart'],
    [30L, 'string', '=', 'mimetypeapplication/vnd.sun.xml.global', 
     'application/vnd.sun.xml.global'],
    # zip works now, after we have it with lower priority than OOo
    [0L, 'string', '=', 'PK\003\004', 'application/zip'],
    [0L, 'string', '=', 'GIF8', 'image/gif'],
    [4L, 'string', '=', 'moov', 'video/quicktime'],
    [4L, 'string', '=', 'mdat', 'video/quicktime'],
    [8L, 'string', '=', 'mp42', 'video/quicktime'],
    [12L, 'string', '=', 'mdat', 'video/quicktime'],
    [36L, 'string', '=', 'mdat', 'video/quicktime'],
    [0L, 'belong', '=', '0x3026b275', 'video/x-ms-asf'],
    [0L, 'string', '=', 'ASF ', 'audio/x-ms-asx'],
    [0L, 'string', '=', '<ASX', 'audio/x-ms-asx'],
    [0L, 'string', '=', '<asx', 'audio/x-ms-asx'],
    [0L, 'string', '=', 'MThd', 'audio/x-midi'],
    [0L, 'string', '=', 'IMPM', 'audio/x-it'],
    [2L, 'string', '=', '-lh0-', 'application/x-lha'],
    [2L, 'string', '=', '-lh1-', 'application/x-lha'],
    [2L, 'string', '=', '-lz4-', 'application/x-lha'],
    [2L, 'string', '=', '-lz5-', 'application/x-lha'],
    [2L, 'string', '=', '-lzs-', 'application/x-lha'],
    [2L, 'string', '=', '-lh\40-', 'application/x-lha'],
    [2L, 'string', '=', '-lhd-', 'application/x-lha'],
    [2L, 'string', '=', '-lh2-', 'application/x-lha'],
    [2L, 'string', '=', '-lh3-', 'application/x-lha'],
    [2L, 'string', '=', '-lh4-', 'application/x-lha'],
    [2L, 'string', '=', '-lh5-', 'application/x-lha'],
    [20L, 'string', '=', '\375\304\247\334', 'application/x-zoo'],
    [0L, 'string', '=', 'StuffIt ', 'application/x-stuffit'],
    [11L, 'string', '=', 'must be converted with BinHex', 'application/mac-binhex40'],
    [102L, 'string', '=', 'mBIN', 'application/x-macbinary'],
    [4L, 'string', '=', 'gtktalog ', 'application/x-gtktalog'],
    [0L, 'string', '=', 'diff ', 'text/x-patch'],
    [0L, 'string', '=', 'Index:', 'text/x-patch'],
    [0L, 'string', '=', '*** ', 'text/x-patch'],
    [0L, 'string', '=', 'Only in ', 'text/x-patch'],
    [0L, 'string', '=', 'Common subdirectories: ', 'text/x-patch'],
    [0L, 'string', '=', 'FONT', 'application/x-font-vfont'],
    [0L, 'string', '=', 'IIN1', 'image/tiff'],
    [0L, 'string', '=', 'MM\x00\x2a', 'image/tiff'],
    [0L, 'string', '=', 'II\x2a\x00', 'image/tiff'],
    [0L, 'string', '=', '\x89PNG', 'image/png'],
    [0L, 'string', '=', '8BPS\ \ \000\000\000\000 &0xffffffff0000ffffffff', 'image/x-psd'],
    [0L, 'string', '=', '#LyX', 'text/x-lyx'],
    [0L, 'string', '=', 'DCMw', 'image/x-dcm'],
    [0L, 'string', '=', 'gimp xcf', 'application/x-gimp-image'],
    [0L, 'belong', '=', '0x59a66a95', 'image/x-sun-raster'],
    [0L, 'belong', '=', '0x01da0000 &0xfcfeffff', 'image/x-sgi'],
    [0L, 'belong', '=', '0xb168de3a', 'image/x-pcx'],
    [0L, 'string', '=', '\x28\x00\x00\x00', 'image/x-dib'],
    [0L, 'string', '=', 'SIMPLE  =', 'image/x-fits'],
    [0L, 'belong', '=', '0x46506978', 'image/x-fpx'],
    [0L, 'belong', '=', '0x00000200', 'image/x-icb'],
    [0L, 'belong', '=', '0x53445058', 'image/x-dpx'],
    [0L, 'string', '=', '[Desktop Entry]', 'application/x-gnome-app-info'],
    [0L, 'string', '=', '[X-GNOME-Metatheme]', 'application/x-gnome-theme'],
    [0L, 'string', '=', '<nautilus_object nautilus_link', 'application/x-nautilus-link'],
    [0L, 'string', '=', 'URL:', 'application/x-gmc-link'],
    [0L, 'string', '=', '/* XPM */', 'image/x-xpixmap'],
    [0L, 'string', '=', '<!DOCTYPE xbel', 'application/xbel'],
    [0L, 'string', '=', '<xbel', 'application/xbel'],
    [0L, 'string', '=', '<!DOCTYPE NETSCAPE-Bookmark-file-1\>', 'application/x-mozilla-bookmarks'],
    [0L, 'string', '=', '<!DOCTYPE NETSCAPE-Bookmark-file-1\>', 'application/x-netscape-bookmarks'],
    [0L, 'string', '=', '<ephy_bookmarks        ', 'application/x-epiphany-bookmarks'],
    [0L, 'string', '=', '<!DOCTYPE svg', 'image/svg'],
    [0L, 'string', '=', '<svg', 'image/svg'],
    [0L, 'string', '=', '<?php', 'application/x-php'],
    [0L, 'string', '=', '<smil\>', 'application/x-smil'],
    [0L, 'string', '=', '<SMIL\>', 'application/x-smil'],
    [0L, 'string', '=', '<!DOCTYPE HTML', 'text/html'],
    [0L, 'string', '=', '<!DOCTYPE html', 'text/html'],
    [0L, 'string', '=', '<!doctype html', 'text/html'],
    [0L, 'string', '=', '<!doctype Html', 'text/html'],
    [0L, 'string', '=', '<!doctype HTML', 'text/html'],
    [10L, 'string', '=', '<HEAD', 'text/html'],
    [10L, 'string', '=', '<head', 'text/html'],
    [16L, 'string', '=', '<TITLE', 'text/html'],
    [16L, 'string', '=', '<title', 'text/html'],
    [10L, 'string', '=', '<html', 'text/html'],
    [0L, 'string', '=', '<HTML', 'text/html'],
    [0L, 'string', '=', '<dia:diagram', 'application/x-dia-diagram'],
    [0L, 'string', '=', '<abiword', 'application/x-abiword'],
    [0L, 'string', '=', '<\!DOCTYPE abiword', 'application/x-abiword'],
    [0L, 'string', '=', 'gmr:Workbook', 'application/x-gnumeric'],
    [0L, 'string', '=', '<?xml', 'text/xml'],
    [0L, 'string', '=', '{\\rtf', 'application/rtf'],
    [0L, 'string', '=', '#!/bin/sh', 'text/x-sh'],
    [0L, 'string', '=', '#!/bin/bash', 'text/x-sh'],
    [0L, 'string', '=', '#!/bin/csh', 'text/x-csh'],
    [0L, 'string', '=', '#!/bin/ksh', 'text/x-ksh'],
    [0L, 'string', '=', '#!/bin/perl', 'text/x-perl'],
    [0L, 'string', '=', '#!/bin/zsh', 'text/x-zsh'],
    [1L, 'string', '=', '/bin/sh', 'text/x-sh'],
    [1L, 'string', '=', '/bin/bash', 'text/x-sh'],
    [1L, 'string', '=', '/bin/csh', 'text/x-csh'],
    [1L, 'string', '=', '/bin/ksh', 'text/x-ksh'],
    [1L, 'string', '=', '/bin/perl', 'text/x-perl'],
    [0L, 'string', '=', 'BEGIN:VCARD', 'text/x-vcard'],
    [0L, 'string', '=', 'BEGIN:VCALENDAR', 'text/calendar'],
    [8L, 'string', '=', 'CDR vrsn', 'application/vnd.corel-draw'],
    [8L, 'string', '=', 'AVI ', 'video/x-msvideo'],
    [0L, 'string', '=', 'MOVI', 'video/x-sgi-movie'],
    [0L, 'string', '=', '.snd', 'audio/basic'],
    [8L, 'string', '=', 'AIFC', 'audio/x-aifc'],
    [8L, 'string', '=', 'AIFF', 'audio/x-aiff'],
    [0L, 'string', '=', '.ra\375', 'audio/x-pn-realaudio'],
    [0L, 'belong', '=', '0x2e7261fd', 'audio/x-pn-realaudio'],
    [0L, 'string', '=', '.RMF', 'audio/x-pn-realaudio'],
    [8L, 'string', '=', 'WAVE', 'audio/x-wav'],
    [8L, 'string', '=', 'WAV ', 'audio/x-wav'],
    [0L, 'string', '=', 'ID3', 'audio/mpeg'],
    [0L, 'string', '=', '0xfff0', 'audio/mpeg'],
    [0L, 'string', '=', '\x00\x00\x01\xba', 'video/mpeg'],
    [8L, 'string', '=', 'CDXA', 'video/mpeg'],
    [0L, 'belong', '=', '0x000001ba', 'video/mpeg'],
    [0L, 'belong', '=', '0x000001b3', 'video/mpeg'],
    [0L, 'string', '=', 'RIFF', 'audio/x-riff'],
    [0L, 'string', '=', 'OggS   ', 'application/ogg'],
    [0L, 'string', '=', 'pnm:\/\/', 'audio/x-real-audio'],
    [0L, 'string', '=', 'rtsp:\/\/', 'audio/x-real-audio'],
    [0L, 'string', '=', 'SIT!', 'application/x-stuffit'],
    [0L, 'string', '=', '\312\376\272\276', 'application/x-java-byte-code'],
    [0L, 'string', '=', 'Joy!', 'application/x-pef-executable'],
    [4L, 'string', '=', '\x11\xAF', 'video/x-fli'],
    [4L, 'string', '=', '\x12\xAF', 'video/x-flc'],
    [0L, 'string', '=', '\x31\xbe\x00\x00', 'application/msword'],
    [0L, 'string', '=', 'PO^Q`', 'application/msword'],
    [0L, 'string', '=', '\376\067\0\043', 'application/msword'],
    [0L, 'string', '=', '\320\317\021\340\241\261', 'application/msword'],
    [0L, 'string', '=', '\333\245-\0\0\0', 'application/msword'],
    [0L, 'string', '=', 'Microsoft Excel 5.0 Worksheet', 'application/vnd.ms-excel'],
    [0L, 'string', '=', 'Biff5', 'application/vnd.ms-excel'],
    [0L, 'string', '=', '*BEGIN SPREADSHEETS    ', 'application/x-applix-spreadsheet'],
    [0L, 'string', '=', '*BEGIN SPREADSHEETS    ', 'application/x-applix-spreadsheet'],
    [0L, 'string', '=', '\x00\x00\x02\x00', 'application/vnd.lotus-1-2-3'],
    [0L, 'belong', '=', '0x00001a00', 'application/vnd.lotus-1-2-3'],
    [0L, 'belong', '=', '0x00000200', 'application/vnd.lotus-1-2-3'],
    [0L, 'string', '=', 'PSID', 'audio/prs.sid'],
    [31L, 'string', '=', 'Oleo', 'application/x-oleo'],
    [0L, 'string', '=', 'FFIL', 'application/x-font-ttf'],
    [65L, 'string', '=', 'FFIL', 'application/x-font-ttf'],
    [0L, 'string', '=', 'LWFN', 'application/x-font-type1'],
    [65L, 'string', '=', 'LWFN', 'application/x-font-type1'],
    [0L, 'string', '=', 'StartFont', 'application/x-font-sunos-news'],
    [0L, 'string', '=', '\x13\x7A\x29', 'application/x-font-sunos-news'],
    [8L, 'string', '=', '\x13\x7A\x2B', 'application/x-font-sunos-news'],
    [0L, 'string', '=', '%!PS-AdobeFont-1.', 'application/x-font-type1'],
    [6L, 'string', '=', '%!PS-AdobeFont-1.', 'application/x-font-type1'],
    [0L, 'string', '=', '%!FontType1-1.', 'application/x-font-type1'],
    [6L, 'string', '=', '%!FontType1-1.', 'application/x-font-type1'],
    [0L, 'string', '=', 'STARTFONT\040', 'application/x-font-bdf'],
    [0L, 'string', '=', '\001fcp', 'application/x-font-pcf'],
    [0L, 'string', '=', 'D1.0\015', 'application/x-font-speedo'],
    [0L, 'string', '=', '\x14\x02\x59\x19', 'application/x-font-libgrx'],
    [0L, 'string', '=', '\xff\x46\x4f\x4e', 'application/x-font-dos'],
    [7L, 'string', '=', '\x00\x45\x47\x41', 'application/x-font-dos'],
    [7L, 'string', '=', '\x00\x56\x49\x44', 'application/x-font-dos'],
    [0L, 'string', '=', '\<MakerScreenFont', 'application/x-font-framemaker'],
    [0L, 'string', '=', '\000\001\000\000\000', 'application/x-font-ttf'],
    [1L, 'string', '=', 'WPC', 'application/x-wordperfect'],
    [0L, 'string', '=', 'ID;', 'text/spreadsheet'],
    [0L, 'string', '=', 'MZ', 'application/x-ms-dos-executable'],
    [0L, 'string', '=', '%!', 'application/postscript'],
    [0L, 'string', '=', 'BZh', 'application/x-bzip'],
    [0L, 'string', '=', '\x1f\x8b', 'application/x-gzip'],
    [0L, 'string', '=', '\037\235', 'application/x-compress'],
    [0L, 'string', '=', '\367\002', 'application/x-dvi'],
    [0L, 'string', '=', '\367\203', 'application/x-font-tex'],
    [0L, 'string', '=', '\367\131', 'application/x-font-tex'],
    [0L, 'string', '=', '\367\312', 'application/x-font-tex'],
    [2L, 'string', '=', '\000\022', 'application/x-font-tex-tfm'],
    [0L, 'string', '=', '\x36\x04', 'application/x-font-linux-psf'],
    [0L, 'string', '=', 'FWS', 'application/x-shockwave-flash'],
    [0L, 'string', '=', 'CWS', 'application/x-shockwave-flash'],
    [0L, 'string', '=', 'NSVf', 'video/x-nsv'],
    [0L, 'string', '=', 'BMxxxx\000\000 &0xffff00000000ffff', 'image/bmp'],
    [0L, 'string', '=', 'Return-Path:', 'message/rfc822'],
    [0L, 'string', '=', 'Path:', 'message/news'],
    [0L, 'string', '=', 'Xref:', 'message/news'],
    [0L, 'string', '=', 'From:', 'message/rfc822'],
    [0L, 'string', '=', 'Article', 'message/news'],
    [0L, 'string', '=', 'Received:', 'message/rfc822'],
    [0L, 'string', '=', '[playlist]', 'audio/x-scpls'],
    [0L, 'string', '=', '[Reference]', 'video/x-ms-asf'],
    [0L, 'string', '=', 'fLaC', 'application/x-flac'],
    [32769L, 'string', '=', 'CD001', 'application/x-iso-image'],
    [37633L, 'string', '=', 'CD001', 'application/x-iso-image'],
    [32776L, 'string', '=', 'CDROM', 'application/x-iso-image'],
    [0L, 'string', '=', 'OTTO', 'application/x-font-otf'],
    [54L, 'string', '=', 'S T O P', 'application/x-ipod-firmware'],
    [0L, 'string', '=', 'BLENDER', 'application/x-blender'],
    [20L, 'string', '=', 'import', 'text/python-source'],
]

magicNumbers = []

def strToNum(n):
    val = 0
    col = long(1)
    if n[:1] == 'x': n = '0' + n
    if n[:2] == '0x':
        # hex
        n = string.lower(n[2:])
        while len(n) > 0:
            l = n[len(n) - 1]
            val = val + string.hexdigits.index(l) * col
            col = col * 16
            n = n[:len(n)-1]
    elif n[0] == '\\':
        # octal
        n = n[1:]
        while len(n) > 0:
            l = n[len(n) - 1]
            if ord(l) < 48 or ord(l) > 57: break
            val = val + int(l) * col
            col = col * 8
            n = n[:len(n)-1]
    else:
        val = string.atol(n)
    return val

class magicTest:
    def __init__(self, offset, t, op, value, msg, mask = None):
        if t.count('&') > 0:
            mask = strToNum(t[t.index('&')+1:])
            t = t[:t.index('&')]
        if type(offset) == type('a'):
            self.offset = strToNum(offset)
        else:
            self.offset = offset
        self.type = t
        self.msg = msg
        self.subTests = []
        self.op = op
        self.mask = mask
        self.value = value

    def test(self, data):
        if self.mask:
            data = data & self.mask
        if self.op == '=':
            if self.value == data: return self.msg
        elif self.op ==  '<':
            pass
        elif self.op ==  '>':
            pass
        elif self.op ==  '&':
            pass
        elif self.op ==  '^':
            pass
        return None

    def compare(self, data):
    #print str([self.type, self.value, self.msg])
        try:
            if self.type == 'string':
                c = ''; s = ''
                for i in range(0, len(self.value)+1):
                    if i + self.offset > len(data) - 1: break
                    s = s + c
                    [c] = struct.unpack('c', data[self.offset + i])
                data = s
            elif self.type == 'short':
                [data] = struct.unpack('h', data[self.offset : self.offset + 2])
            elif self.type == 'leshort':
                [data] = struct.unpack('<h', data[self.offset : self.offset + 2])
            elif self.type == 'beshort':
                [data] = struct.unpack('>H', data[self.offset : self.offset + 2])
            elif self.type == 'long':
                [data] = struct.unpack('l', data[self.offset : self.offset + 4])
            elif self.type == 'lelong':
                [data] = struct.unpack('<l', data[self.offset : self.offset + 4])
            elif self.type == 'belong':
                [data] = struct.unpack('>l', data[self.offset : self.offset + 4])
            else:
                #print 'UNKNOWN TYPE: ' + self.type
                pass
        except:
            return None

#    print str([self.msg, self.value, data])
        return self.test(data)


def guessMime(data):
    for test in magicNumbers:
        m = test.compare(data)
        if m: 
            return m
    # no matching, magic number.
    return

#import sys
for m in magic:
    magicNumbers.append(magicTest(m[0], m[1], m[2], m[3], m[4]))



"""some common utilities
"""

FB_REGISTRY = None

# base class
from ExtensionClass import Base
from Acquisition import aq_base

# logging function
from zLOG import LOG, INFO
def log(msg, severity=INFO, id='PortalTransforms'):
    LOG(id, severity, msg)

# directory where template for the ZMI are located
import os.path
_www = os.path.join(os.path.dirname(__file__), 'www')
skins_dir = os.path.join(os.path.dirname(__file__), 'skins')

# list and dict classes to use
from Globals import PersistentMapping as DictClass
try:
    from ZODB.PersistentList import PersistentList as ListClass
except ImportError:
    from persistent.list import PersistentList as ListClass

# interfaces
try:
    # Zope >= 2.6
    from Interface import Interface, Attribute
except ImportError:
    # Zope < 2.6
    from Interface import Base as Interface, Attribute

def implements(object, interface):
    return interface.isImplementedBy(object)

# getToolByName
from Products.CMFCore.utils import getToolByName as _getToolByName
_marker = []

def getToolByName(context, name, default=_marker):
    global FB_REGISTRY
    tool = _getToolByName(context, name, default)
    if name == 'mimetypes_registry' and tool is default:
        if FB_REGISTRY is None:
            from Products.MimetypesRegistry.MimeTypesRegistry \
                 import MimeTypesRegistry
            FB_REGISTRY = MimeTypesRegistry()
        tool = FB_REGISTRY
    return tool

from zExceptions import BadRequest

__all__ = ('Base', 'log', 'DictClass', 'ListClass', 'getToolByName', 'aq_base',
           'Interface', 'Attribute', 'implements', 'skins_dir', '_www',
           'BadRequest', )



import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))
from Testing import ZopeTestCase
from Products.Archetypes.tests.common import *

from Products.MimetypesRegistry.mime_types import text_plain, text_xml, application_octet_stream
from utils import input_file_path

class TestMimeTypesclass(ArcheSiteTestCase):

    def afterSetUp(self):
        ArcheSiteTestCase.afterSetUp(self)
        self.registry = self.portal.mimetypes_registry

    def testClassify(self):
        reg = self.registry
        c = reg._classifiers()
        self.failUnless(c[0].name().startswith("Extensible Markup Language"),
                        c[0].name())

        #Real XML
        data = "<?xml version='1.0'?><foo>bar</foo>"
        mt = reg.classify(data)
        self.failUnless(isinstance(mt, text_xml), str(mt))

        #Passed in MT
        mt = reg.classify(data, mimetype="text/plain")
        self.failUnless(isinstance(mt, text_plain), str(mt))

        #Passed in filename
        mt = reg.classify(data, filename="test.xml")
        self.failUnless(isinstance(mt, text_xml), str(mt))
        mt = reg.classify(data, filename="test.jpg")
        self.failUnlessEqual(str(mt), 'image/jpeg')

        # use xml classifier
        mt = reg.classify('<?xml ?>')
        self.failUnless(isinstance(mt, text_xml), str(mt))

        # test no data return default
        mt = reg.classify('')
        self.failUnless(isinstance(mt, text_plain), str(mt))
        reg.defaultMimetype = 'text/xml'
        mt = reg.classify('')
        self.failUnless(isinstance(mt, text_xml), str(mt))

        # test unclassifiable data and no stream flag (filename)
        mt = reg.classify('xxx')
        self.failUnless(isinstance(mt, text_plain), str(mt))

        # test unclassifiable data and file flag
        mt = reg.classify('baz', filename='xxx')
        self.failUnless(isinstance(mt, application_octet_stream), str(mt))

    def testExtension(self):
        reg = self.registry
        data = "<foo>bar</foo>"
        mt = reg.lookupExtension(filename="test.xml")
        self.failUnless(isinstance(mt, text_xml), str(mt))

        mt = reg.classify(data, filename="test.foo")
        self.failUnless(isinstance(mt, application_octet_stream), str(mt))

        mt = reg.classify(data, filename="test.tgz")
        self.failUnlessEqual(str(mt), 'application/x-tar')

        mt = reg.classify(data, filename="test.tar.gz")
        self.failUnlessEqual(str(mt), 'application/x-tar')

        mt = reg.classify(data, filename="test.pdf.gz")
        self.failUnlessEqual(str(mt), 'application/pdf')

    def testLookup(self):
        reg = self.registry
        mt = reg.lookup('text/plain')
        self.failUnless(isinstance(mt[0], text_plain), str(mt[0]))

        mt = reg.lookup('text/notexistent')
        self.failUnlessEqual(mt, ())

    def testAdaptMt(self):
        data, filename, mt = self.registry('bar', mimetype='text/xml')
        # this test that data has been adaped and file seeked to 0
        self.failUnlessEqual(data, 'bar')
        self.failUnlessEqual(filename, None)
        self.failUnless(isinstance(mt, text_xml), str(mt))

    def testAdaptFile(self):
        file = open(input_file_path("rest1.rst"))
        data, filename, mt = self.registry(file)
        # this test that data has been adaped and file seeked to 0
        self.failUnlessEqual(data, file.read())
        file.close()
        self.failUnlessEqual(filename, "rest1.rst")
        self.assertEqual(str(mt), 'text/x-rst')

    def testAdaptData(self):
        data, filename, mt = self.registry('<?xml ?>')
        # this test that data has been adaped and file seeked to 0
        self.failUnlessEqual(data, '<?xml ?>')
        self.failUnlessEqual(filename, None)
        self.failUnless(isinstance(mt, text_xml), str(mt))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMimeTypesclass))
    return suite

if __name__ == '__main__':
    framework()



from xml.dom import minidom, XML_NAMESPACE
import os

from Products.MimetypesRegistry.MimeTypeItem import MimeTypeItem
from Products.MimetypesRegistry.common import MimeTypeException

#shared mime info name space
SMINS = 'http://www.freedesktop.org/standards/shared-mime-info'

DIR = os.path.dirname(__file__)
SMI_NAME = "freedesktop.org.xml"
SMI_FILE = os.path.join(DIR, SMI_NAME)

def readSMIFile(infofile):
    """Reads a shared mime info XML file
    
    Bases on the reader from kio-transforms
    Project:    Kio Transforms
    Client:	    Isia standing group
    Authors:    Joe Geldart (jgeldart) -- jgeldart@netalleynetworks.com
                Michael Zeltner (mzeltner) -- mzeltner@netalleynetworks.com
    """
    dom = minidom.parse(infofile)
    
    results = []
    
    # For each mime-type node...
    for mtype in dom.getElementsByTagName('mime-type'):
        # Get the MIME type
        type = mtype.getAttribute('type')
        children = [ c for c in mtype.childNodes if c.nodeType == c.ELEMENT_NODE ]
        
        # For each comment node...
        commentnodes = [ com for com in children if com.tagName == 'comment']
        comments = {}
        for com in commentnodes:
            lang = com.getAttributeNS(XML_NAMESPACE, 'lang') or 'en'
            comments[lang] = ''.join([n.nodeValue for n in com.childNodes]).strip()

        # For each glob node...
        globnodes = [ glb for glb in children if glb.tagName == 'glob']
        globs = []
        for glob in globnodes:
            globs.append(glob.getAttribute('pattern') or None)

        results.append({'type' : type,
                        'comments' : comments,
                        'globs' : globs,
                       })
    
    # Unlink reference cycles (for garbage collection)
    dom.unlink()
    
    return results

mimetypes = readSMIFile(SMI_FILE)

def initialize(registry):
    #Find things that are not in the specially registered mimetypes
    #and add them using some default policy, none of these will impl
    #iclassifier
    global mimetypes
    for res in mimetypes:
        mt = str(res['type'])

        # check the mime type
        try:
            mto =  registry.lookup(mt)
        except MimeTypeException:
            # malformed MIME type
            continue

        name = str(res['comments'].get(u'en', mt))

        # build a list of extensions 
        exts = []
        for ext in res['globs']:
            if ext.startswith('.'):
                ext = ext[1:]
            if registry.lookupExtension(ext):
                continue
            else:
                exts.append(ext)

        if mto:
            # The mimetype was already registered but not the extension
            mto = mto[0]
            if not ext in mto.extensions:
                registry.register_extension(ext, mto)
                mto.extensions += (ext, )
            continue
        isBin = mt.split('/', 1)[0] != "text"
        mti = MimeTypeItem(name, (mt,), exts, isBin)
        registry.register(mti)



import os.path
__version__ = open(os.path.join(__path__[0], 'version.txt')).read().strip()

from Products.MimetypesRegistry import MimeTypesRegistry
from Products.MimetypesRegistry.common import skins_dir

GLOBALS = globals()
PKG_NAME = 'MimetypesRegistry'

tools = (
    MimeTypesRegistry.MimeTypesRegistry,
    )

# XXX backward compatibility tricks to make old PortalTransform based Mimetypes
# running (required)
import sys
from Products.MimetypesRegistry import mime_types
sys.modules['Products.PortalTransforms.mime_types'] = mime_types

from Products.MimetypesRegistry import MimeTypeItem
sys.modules['Products.PortalTransforms.MimeTypeItem'] = MimeTypeItem

from Products.MimetypesRegistry import MimeTypeItem
sys.modules['Products.PortalTransforms.zope.MimeTypeItem'] = MimeTypeItem
sys.modules['Products.MimetypesRegistry.zope.MimeTypeItem'] = MimeTypeItem

def initialize(context):
    from Products.CMFCore.DirectoryView import registerDirectory
    registerDirectory(skins_dir, GLOBALS)

    from Products.CMFCore import utils
    utils.ToolInit("%s Tool" % PKG_NAME, tools=tools,
                   product_name=PKG_NAME,
                   icon="tool.gif",
                   ).initialize(context)



import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.Archetypes.tests.common import *

from Products.MimetypesRegistry.mime_types.magic import guessMime
from utils import input_file_path

samplefiles = [
    ('OOoWriter', 'application/vnd.sun.xml.writer'),
    ('OOoCalc', 'application/vnd.sun.xml.calc'),
    ('sxw-ooo-trolltech', 'application/vnd.sun.xml.writer'), # file from limi
    ('simplezip', 'application/zip'),
]

class TestGuessMagic(ArcheSiteTestCase):
    
    def afterSetUp(self):
        ArcheSiteTestCase.afterSetUp(self)
        self.registry = self.portal.mimetypes_registry


    def test_guessMime(self):
        for filename, expected in samplefiles:
            file = open(input_file_path(filename))
            data = file.read()
            file.close()
            
            # use method direct
            got = guessMime(data)
            self.failUnlessEqual(got, expected)
            
            # use mtr-tool
            got_from_tool = self.registry.classify(data)
            self.failUnlessEqual(got_from_tool, expected)            
            
            # now cut it to the first 8k if greater
            if len(data) > 8192:
                data=data[:8192]
                got_cutted = self.registry.classify(data)
                self.failUnlessEqual(got_cutted, expected)          


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestGuessMagic))
    return suite

if __name__ == '__main__':
    framework()



import re
import encodings
from Products.MimetypesRegistry.common import log

EMACS_ENCODING_RGX = re.compile('[^#]*[#\s]*-\*-\s*coding: ([^\s]*)\s*-\*-\s*')
VIM_ENCODING_RGX = re.compile('[^#]*[#\s]*vim:fileencoding=\s*([^\s]*)\s*')
XML_ENCODING_RGX = re.compile('<\?xml version=[^\s]*\s*encoding=([^\s]*)\s*\?>')
CHARSET_RGX = re.compile('charset=([^\s"]*)')

def guess_encoding(buffer):
    """Better guess encoding method
    
    It checks if python supports the encoding
    """
    encoding = _guess_encoding(buffer)
    # step 1: if the encoding was detected, use the lower() because python
    # is using lower case names for encodings
    if encoding and isinstance(encoding, basestring):
        #encoding = encoding.lower()
        pass
    else:
        return None
    # try to find an encoding function for the encoding
    # if None is returned or an exception is raised the encoding is invalid
    try:
        result = encodings.search_function(encoding.lower())
    except:
        # XXX log
        result = None
    
    if result:
        # got a valid encoding
        return encoding
    else:
        return None
    

def _guess_encoding(buffer):
    """try to guess encoding from a buffer

    FIXME: it could be mime type driven but it seems less painful like that
    """
    assert type(buffer) is type(''), type(buffer)
    # default to ascii on empty buffer
    if not buffer:
        return 'ascii'

    # check for UTF-8 byte-order mark
    if buffer.startswith('\xef\xbb\xbf'):
        return 'UTF-8'

    first_lines = buffer.split('\n')[:2]
    for line in first_lines:
        # check for emacs encoding declaration
        m = EMACS_ENCODING_RGX.match(line)
        if m is not None:
            return m.group(1)
        # check for vim encoding declaration
        m = VIM_ENCODING_RGX.match(line)
        if m is not None:
            return m.group(1)

    # check for xml encoding declaration
    if first_lines[0].startswith('<?xml'):
        m = XML_ENCODING_RGX.match(first_lines[0])
        if m is not None:
            return m.group(1)[1:-1]
        # xml files with no encoding declaration default to UTF-8
        return 'UTF-8'

    # try to get charset declaration
    # FIXME: should we check it's html before ?
    m = CHARSET_RGX.search(buffer)
    if m is not None:
        return m.group(1)
    return None



"""some common utilities
"""
from time import time
from types import UnicodeType, StringType

STRING_TYPES = (UnicodeType, StringType)

class MimeTypeException(Exception):
    pass

# logging function
from zLOG import LOG, INFO
def log(msg, severity=INFO, id='MimetypesRegistry'):
    LOG(id, severity, msg)

# directory where template for the ZMI are located
import os.path
_www = os.path.join(os.path.dirname(__file__), 'www')
skins_dir = os.path.join(os.path.dirname(__file__), 'skins')



#
# Runs all tests in the current directory
#
# Execute like:
#   python runalltests.py
#
# Alternatively use the testrunner: 
#   python /path/to/Zope/utilities/testrunner.py -qa
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py')) 

import unittest
TestRunner = unittest.TextTestRunner
suite = unittest.TestSuite()

tests = os.listdir(os.curdir)
tests = [n[:-3] for n in tests if n.startswith('test') and n.endswith('.py')]

for test in tests:
    m = __import__(test)
    if hasattr(m, 'test_suite'):
        suite.addTest(m.test_suite())

if __name__ == '__main__':
    TestRunner().run(suite)




from Products.MimetypesRegistry.interfaces import IClassifier
from Products.MimetypesRegistry.MimeTypeItem import MimeTypeItem
from Products.MimetypesRegistry.common import MimeTypeException

from types import InstanceType
import re

class text_plain(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Plain Text"
    mimetypes  = ('text/plain',)
    extensions = ('txt',)
    binary     = 0

class text_pre_plain(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Pre-formatted Text (<pre>)"
    mimetypes  = ('text/plain-pre',)
    extensions = ()
    binary     = 0

class text_structured(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Structured Text"
    mimetypes  = ('text/structured',)
    extensions = ('stx',)
    binary     = 0

class text_rest(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "reStructured Text"
    mimetypes  = ("text/x-rst", "text/restructured",)
    extensions = ("rst", "rest", "restx") #txt?
    binary     = 0

class text_python(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Python Source"
    mimetypes  = ("text/python-source", "text/x-python",)
    extensions = ("py",)
    binary     = 0

class text_wiki(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Wiki text"
    mimetypes  = ("text/wiki",)
    extensions = ()
    binary     = 0

class application_rtf(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = 'Rich Text Format (RTF)'
    mimetypes  = ('application/rtf',)
    extensions = ('rtf',)
    binary     = 1

class application_msword(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Microsoft Word Document"
    mimetypes  = ('application/msword',)
    extensions = ('doc',)
    binary     = 1

class text_xml(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__ + (IClassifier,)

    __name__   = "Extensible Markup Language (XML)"
    mimetypes  = ('text/xml',)
    extensions = ('xml',)
    binary     = 0

    def classify(self, data):
        m = re.search('^\s*<?xml.*?>', data)
        if m:
            return 1 # True
        return None  # False

class application_octet_stream(MimeTypeItem):
    """we need to be sure this one exists"""
    __name__   = "Octet Stream"
    mimetypes = ('application/octet-stream',)
    binary     = 1
    extensions = ()

class text_html(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "HTML"
    mimetypes  = ('text/html',)
    extensions = ('html', 'htm')
    binary     = 0

class text_html_safe(MimeTypeItem):

    __implements__ = MimeTypeItem.__implements__
    __name__   = "Safe HTML"
    mimetypes  = ('text/x-html-safe',)
    extensions = ()
    binary     = 0

reg_types = [
    text_plain,
    text_pre_plain,
    application_msword,
    text_xml,
    text_structured,
    text_rest,
    text_python,
    text_wiki,
    application_octet_stream,
    application_rtf,
    text_html,
    text_html_safe,
    ]

def initialize(registry):
    for mt in reg_types:
        if type(mt) != InstanceType:
            mt = mt()
        registry.register(mt)

__all__ = tuple([cls.__name__ for cls in reg_types])



from Products.MimetypesRegistry.MimeTypeItem import MimeTypeItem, guess_icon_path
from Products.MimetypesRegistry.common import MimeTypeException

import mimetypes as pymimetypes

# don't register the mimetype from python mimetypes if matching on of this
# extensions.
skip_extensions = (
    )

def initialize(registry):
    #Find things that are not in the specially registered mimetypes
    #and add them using some default policy, none of these will impl
    #iclassifier
    for ext, mt in pymimetypes.types_map.items():
        if ext[0] == '.':
            ext = ext[1:]
        
        if registry.lookupExtension(ext):
            continue
        if ext in skip_extensions:
            continue

        try:
            mto =  registry.lookup(mt)
        except MimeTypeException:
            # malformed MIME type
            continue
        if mto:
            mto = mto[0]
            if not ext in mto.extensions:
                registry.register_extension(ext, mto)
                mto.extensions += (ext, )
                # here we guess icon path again, to find icon match the new ext
                mto.icon_path = guess_icon_path(mto)
            continue
        isBin = mt.split('/', 1)[0] != "text"
        registry.register(MimeTypeItem(mt, (mt,), (ext,), isBin))



import os

from Acquisition import Explicit
from OFS.SimpleItem import Item
from AccessControl import ClassSecurityInfo
from Globals import Persistent, InitializeClass
from Products.CMFCore import CMFCorePermissions

from Products.MimetypesRegistry.interfaces import IMimetype
from Products.MimetypesRegistry.common import MimeTypeException


class MimeTypeItem(Persistent, Explicit, Item):
    security = ClassSecurityInfo()
    __implements__ = (IMimetype,)

    def __init__(self, name='', mimetypes=None, extensions=None, binary=None,
                 icon_path=''):
        if name:
            self.__name__ = self.id = name
        if mimetypes is not None:
            self.mimetypes = mimetypes
        if extensions is not None:
            self.extensions = extensions
        if binary is not None:
            self.binary = binary
        self.icon_path = icon_path or guess_icon_path(self)

    def __str__(self):
        return self.normalized()

    def __repr__(self):
        return "<mimetype %s>" % self.mimetypes[0]

    def __cmp__(self, other):
        try:
            if isinstance(other, mimetype):
                other = other.normalized()
        except:
            pass
        return not (other in self.mimetypes)

    def __hash__(self):
        return hash(self.name())

    security.declarePublic('name')
    def name(self):
        """ The name of this object """
        return self.__name__

    security.declarePublic('major')
    def major(self):
        """ return the major part of the RFC-2046 name for this mime type """
        return self.normalized().split('/', 1)[0]

    security.declarePublic('minor')
    def minor(self):
        """ return the minor part of the RFC-2046 name for this mime type """
        return self.normalized().split('/', 1)[1]

    security.declarePublic('normalized')
    def normalized(self):
        """ return the main RFC-2046 name for this mime type

        e.g. if this object has names ('text/restructured', 'text-x-rst')
        then self.normalized() will always return the first form.
        """
        return self.mimetypes[0]

    security.declareProtected(CMFCorePermissions.ManagePortal, 'edit')
    def edit(self, name, mimetypes, extensions, icon_path, binary=0,
             REQUEST=None):
        """edit this mime type"""
        # if mimetypes and extensions are string instead of lists, split them on new lines
        if type(mimetypes) in (type(''), type(u'')):
            mimetypes = [mts.strip() for mts in mimetypes.split('\n') if mts.strip()]
        if type(extensions) in (type(''), type(u'')):
            extensions = [mts.strip() for mts in extensions.split('\n') if mts.strip()]
        self.__name__ = self.id = name
        self.mimetypes = mimetypes
        self.extensions = extensions
        self.binary = binary
        self.icon_path = icon_path
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

InitializeClass(MimeTypeItem)


ICONS_DIR = os.path.join(os.path.dirname(__file__), 'skins', 'mimetypes_icons')

def guess_icon_path(mimetype, icons_dir=ICONS_DIR, icon_ext='png'):
    if mimetype.extensions:
        for ext in mimetype.extensions:
            icon_path = '%s.%s' % (ext, icon_ext)
            if os.path.exists(os.path.join(icons_dir, icon_path)):
                return icon_path
    icon_path = '%s.png' % mimetype.major()
    if os.path.exists(os.path.join(icons_dir, icon_path)):
        return icon_path
    return 'unknown.png'



import mtr_mimetypes
import py_mimetypes
import smi_mimetypes
import suppl_mimetypes
import magic

from mtr_mimetypes import *

def initialize(registry):
    mtr_mimetypes.initialize(registry)
    smi_mimetypes.initialize(registry)
    suppl_mimetypes.initialize(registry)
    py_mimetypes.initialize(registry)



from Interface import Interface, Attribute

class IMimetype(Interface):
    """Specification for dealing with mimetypes RFC-2046 style"""

#     mimetypes = Attribute("List of mimetypes in the RFC-2046 format")
#     extensions = Attribute("""List of extensions mapped to this
#     mimetype w/o the leading .""")

#     binary = Attribute("""Boolean indicating if the mimetype should be
#     treated as binary (and not human readable)""")

    def name(self):
        """return the Human readable name of the mimetype"""

    def major(self):
        """ return the major part of the RFC-2046 name for this mime type """

    def minor(self):
        """ return the minor part of the RFC-2046 name for this mime type """

    def normalized(self):
        """ return the main RFC-2046 name for this mime type

        e.g. if this object has names ('text/restructured', 'text-x-rst')
        then self.normalized() will always return the first form.
        """

class IClassifier(Interface):
    """Optional mixin interface for imimetype, code to test if the
    mimetype is present in data
    """
    def classify(data):
        """ boolean indicating if the data fits the mimetype"""


class ISourceAdapter(Interface):

    def __call__(data, **kwargs):
        """convert data to unicode, may take optional kwargs to aid in
        conversion"""


class IMimetypesRegistry(Interface):

    def classify(data, mimetype=None, filename=None):
        """return a content type for this data or None
        None should rarely be returned as application/octet can be
        used to represent most types
        """

    def lookup(mimetypestring):
        """Lookup for imimetypes object matching mimetypestring

        mimetypestring may have an empty minor part or containing a wildcard (*)
        mimetypestring may be an imimetype object (in this case it will be
        returned unchanged, else it should be a RFC-2046 name

        return a list of mimetypes objects associated with the RFC-2046 name
        return an empty list if no one is known.
        """

    def lookupExtension(filename):
        """ return the mimetypes object associated with the file's extension
        return None if it is not known.

        filename maybe a file name like 'content.txt' or an extension like 'rest'
        """

    def mimetypes():
        """return all defined mime types, each one implements at least imimetype
        """

    def list_mimetypes():
        """return all defined mime types, as string"""



from Products import MimetypesRegistry as PRODUCT
import os.path

version=PRODUCT.__version__
modname=PRODUCT.__name__

# (major, minor, patchlevel, release info) where release info is:
# -99 for alpha, -49 for beta, -19 for rc and 0 for final
# increment the release info number by one e.g. -98 for alpha2

major, minor, bugfix =  version.split('.')[:3]
bugfix, release = bugfix.split('-')[:2]

relinfo=-99 #alpha
if 'beta' in release:
    relinfo=-49
if 'rc' in release:
    relinfo=-19
if 'final' in release:
    relinfo=0

numversion = (int(major), int(minor), int(bugfix), relinfo)

at_versions = (
    '1.3.0-final',
    '1.3.1-rc1',
    '1.3.1-rc2',
    '1.3.1-rc3',
    '1.3.1-rc4',
    '1.3.1-final',
    '1.3.2-rc1',
    '1.3.2-final',
    '1.3.3-rc1',
    '1.3.3-rc2',
    '1.3.3-rc3',
    '1.3.3-final',
    '1.3.4-beta1',
    '1.3.4-beta2',
    '1.3.4-rc1',
    '1.3.4-rc2',
    '1.3.4-rc3',
    '1.3.4-final',
    ###MARKERFORATRELEASESCRIPT###
    )

license = 'BSD like'
license_text = open(os.path.join(PRODUCT.__path__[0], 'LICENSE.txt')).read()
copyright = '''Copyright (c) 2003 LOGILAB S.A. (Paris, FRANCE)'''

author = "Archetypes developement team"
author_email = "archetypes-devel@lists.sourceforge.net"

short_desc = "MIME types registry for the CMF"
long_desc = """This package provides a new CMF tools in order to
make MIME types guessings. You will find more info in the package's
README and docs directory.
.
It's part of the Archetypes project, but the only requirement to use it
is to have a CMF based site. If you are using Archetypes, this package
replaces the transform package.
.
Notice this package can also be used as a standalone Python package. If
you've downloaded the Python distribution, you can't make it a Zope
product since Zope files have been removed from this distribution.
"""

web = "http://www.sourceforge.net/projects/archetypes"
ftp = ""
mailing_list = "archetypes-devel@lists.sourceforge.net"

debian_name = "zope-cmfmtr"
debian_maintainer = "Christian Heimes"
debian_maintainer_email = "heimes@faho.rwth-aachen.de"
debian_handler = "zope"



import os
from types import UnicodeType

from OFS.Folder import Folder
from OFS.content_types import guess_content_type
from Globals import InitializeClass
from Acquisition import aq_parent
from Acquisition import aq_base
from Globals import PersistentMapping
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.TypesTool import FactoryTypeInformation
from Products.CMFCore.utils import UniqueObject
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.MimetypesRegistry.interfaces import ISourceAdapter
from Products.MimetypesRegistry.interfaces import IMimetypesRegistry
from Products.MimetypesRegistry.interfaces import IMimetype
from Products.MimetypesRegistry.interfaces import IClassifier
from Products.MimetypesRegistry.MimeTypeItem import MimeTypeItem
from Products.MimetypesRegistry.mime_types import initialize
from Products.MimetypesRegistry.mime_types import magic
from Products.MimetypesRegistry.common import log
from Products.MimetypesRegistry.common import MimeTypeException
from Products.MimetypesRegistry.common import STRING_TYPES
from Products.MimetypesRegistry.common import _www
from Products.MimetypesRegistry.encoding import guess_encoding
from Products.MimetypesRegistry.common import log


suffix_map = {
    'tgz': '.tar.gz',
    'taz': '.tar.gz',
    'tz': '.tar.gz',
    }

encodings_map = {
    'gz': 'gzip',
    'Z': 'compress',
    }

class MimeTypesRegistry(UniqueObject, ActionProviderBase, Folder):
    """Mimetype registry that deals with
    a) registering types
    b) wildcarding of rfc-2046 types
    c) classifying data into a given type
    """

    __implements__ = (IMimetypesRegistry, ISourceAdapter)

    id        = 'mimetypes_registry'
    meta_type = 'MimeTypes Registry'
    isPrincipiaFolderish = 1 # Show up in the ZMI

    meta_types = all_meta_types = (
        { 'name'   : 'MimeType',
          'action' : 'manage_addMimeTypeForm'},
        )

    manage_options = (
        ( { 'label'   : 'MimeTypes',
            'action' : 'manage_main'},) +
        Folder.manage_options[2:]
        )

    manage_addMimeTypeForm = PageTemplateFile('addMimeType', _www)
    manage_main = PageTemplateFile('listMimeTypes', _www)
    manage_editMimeTypeForm = PageTemplateFile('editMimeType', _www)

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.ManagePortal, 'register')
    security.declareProtected(CMFCorePermissions.ManagePortal, 'unregister')
    security.declarePublic('mimetypes')
    security.declarePublic('list_mimetypes')
    security.declarePublic('lookup')
    security.declarePublic('lookupExtension')
    security.declarePublic('classify')

    # FIXME
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self,):
        self.encodings_map = encodings_map.copy()
        self.suffix_map = suffix_map.copy()
        # Major key -> minor IMimetype objects
        self._mimetypes  = PersistentMapping()
        # ext -> IMimetype mapping
        self.extensions = PersistentMapping()

        self.manage_addProperty('defaultMimetype', 'text/plain', 'string')
        self.manage_addProperty('unicodePolicies', 'strict ignore replace', 'tokens')
        self.manage_addProperty('unicodePolicy', 'unicodePolicies', 'selection')
        self.manage_addProperty('fallbackEncoding', 'latin1', 'string')

        # initialize mime types
        initialize(self)
        self._new_style_mtr = 1

    def register(self, mimetype):
        """ Register a new mimetype

        mimetype must implement IMimetype
        """
        mimetype = aq_base(mimetype)
        assert IMimetype.isImplementedBy(mimetype)
        for t in mimetype.mimetypes:
            major, minor = split(t)
            if not major or not minor or minor == '*':
                raise MimeTypeException('Can\'t register mime type %s' % t)
            group = self._mimetypes.setdefault(major, PersistentMapping())
            if group.has_key(minor):
                if group.get(minor) != mimetype:
                    log('Warning: redefining mime type %s (%s)' % (t, mimetype.__class__))
            group[minor] = mimetype
        for extension in mimetype.extensions:
            self.register_extension(extension, mimetype)

    def register_extension(self, extension, mimetype):
        """ Associate a file's extension to a IMimetype

        extension is a string representing a file extension (not
        prefixed by a dot) mimetype must implement IMimetype
        """
        mimetype = aq_base(mimetype)
        if self.extensions.has_key(extension):
            if self.extensions.get(extension) != mimetype:
                log('Warning: redefining extension %s from %s to %s' % (
                    extension, self.extensions[extension], mimetype))
        #we don't validate fmt yet, but its ["txt", "html"]
        self.extensions[extension] = mimetype


    def unregister(self, mimetype):
        """ Unregister a new mimetype

        mimetype must implement IMimetype
        """
        assert IMimetype.isImplementedBy(mimetype)
        for t in mimetype.mimetypes:
            major, minor = split(t)
            group = self._mimetypes.get(major, {})
            if group.get(minor) == mimetype:
                del group[minor]
        for e in mimetype.extensions:
            if self.extensions.get(e) == mimetype:
                del self.extensions[e]


    def mimetypes(self):
        """Return all defined mime types, each one implements at least
        IMimetype
        """
        res = {}
        for g in self._mimetypes.values():
            for mt in g.values():
                res[mt] =1
        return [ aq_base(mtitem) for mtitem in res.keys() ]

    def list_mimetypes(self):
        """Return all defined mime types, as string"""
        return [str(mt) for mt in self.mimetypes()]

    def lookup(self, mimetypestring):
        """Lookup for IMimetypes object matching mimetypestring

        mimetypestring may have an empty minor part or containing a
        wildcard (*) mimetypestring may and IMimetype object (in this
        case it will be returned unchanged

        Return a list of mimetypes objects associated with the
        RFC-2046 name return an empty list if no one is known.
        """
        if IMimetype.isImplementedBy(mimetypestring):
            return (aq_base(mimetypestring), )
        __traceback_info__ = (repr(mimetypestring), str(mimetypestring))
        major, minor = split(str(mimetypestring))
        group = self._mimetypes.get(major, {})
        if not minor or minor == '*':
            res = group.values()
        else:
            res = group.get(minor)
            if res:
                res = (res,)
            else:
                return ()
        return tuple([ aq_base(mtitem) for mtitem  in res ])

    def lookupExtension(self, filename):
        """Lookup for IMimetypes object matching filename

        Filename maybe a file name like 'content.txt' or an extension
        like 'rest'

        Return an IMimetype object associated with the file's
        extension or None
        """
        if filename.find('.') != -1:
            base, ext = os.path.splitext(filename)
            ext = ext[1:] # remove the dot
            while self.suffix_map.has_key(ext):
                base, ext = os.path.splitext(base + self.suffix_map[ext])
                ext = ext[1:] # remove the dot
        else:
            ext = filename
            base = None

        # XXX This code below make no sense and may break because base
        # isn't defined.
        if self.encodings_map.has_key(ext) and base:
            encoding = self.encodings_map[ext]
            base, ext = os.path.splitext(base)
            ext = ext[1:] # remove the dot
        else:
            encoding = None
        return aq_base(self.extensions.get(ext))

    def _classifiers(self):
        return [mt for mt in self.mimetypes() if IClassifier.isImplementedBy(mt)]

    def classify(self, data, mimetype=None, filename=None):
        """Classify works as follows:
        1) you tell me the rfc-2046 name and I give you an IMimetype
           object
        2) the filename includes an extension from which we can guess
           the mimetype
        3) we can optionally introspect the data
        4) default to self.defaultMimetype if no data was provided
           else to application/octet-stream of no filename was provided,
           else to text/plain

        Return an IMimetype object
        """
        mt = None
        if mimetype:
            mt = self.lookup(mimetype)
            if mt:
                mt = mt[0]
        elif filename:
            mt = self.lookupExtension(filename)
        if data and not mt:
            for c in self._classifiers():
                if c.classify(data):
                    mt = c
                    break
            if not mt:
                mstr = magic.guessMime(data)
                if mstr:
                    mt = self.lookup(mstr)[0]
        if not mt:
            if not data:
                mt = self.lookup(self.defaultMimetype)[0]
            elif filename:
                mt = self.lookup('application/octet-stream')[0]
            else:
                failed = 'text/x-unknown-content-type'
                filename = filename or ''
                data = data or ''
                ct, enc = guess_content_type(filename, data, None)
                if ct == failed:
                    ct = 'text/plain'
                mt = self.lookup(ct)[0]

        # Remove acquisition wrappers
        return aq_base(mt)

    def __call__(self, data, **kwargs):
        """ Return a triple (data, filename, mimetypeobject) given
        some raw data and optional paramters

        method from the isourceAdapter interface
        """
        mimetype = kwargs.get('mimetype', None)
        filename = kwargs.get('filename', None)
        encoding = kwargs.get('encoding', None)
        mt = None
        if hasattr(data, 'filename'):
            filename = os.path.basename(data.filename)
        elif hasattr(data, 'name'):
            filename = os.path.basename(data.name)

        if hasattr(data, 'read'):
            _data = data.read()
            if hasattr(data, 'seek'):
                data.seek(0)
            data = _data

        # We need to figure out if data is binary and skip encoding if
        # it is
        mt = self.classify(data, mimetype=mimetype, filename=filename)

        if not mt.binary and not type(data) is UnicodeType:
            # if no encoding specified, try to guess it from data
            if encoding is None:
                encoding = self.guess_encoding(data)

            # ugly workaround for
            # https://sourceforge.net/tracker/?func=detail&aid=1068001&group_id=75272&atid=543430
            # covered by
            # https://sourceforge.net/tracker/?func=detail&atid=355470&aid=843590&group_id=5470
            # dont remove this code unless python is fixed.
            if encoding is "macintosh":
                encoding = 'mac_roman'

            try:
                try:
                    data = unicode(data, encoding, self.unicodePolicy)
                except (ValueError, LookupError):
                    # wrong unicodePolicy
                    data = unicode(data, encoding)
            except:
                data = unicode(data, self.fallbackEncoding)

        return (data, filename, aq_base(mt))

    def guess_encoding(self, data):
        """ Try to guess encoding from a text value if no encoding
        guessed, used the default charset from site properties (Zope)
        with a fallback to UTF-8 (should never happen with correct
        site_properties, but always raise Attribute error without
        Zope)
        """
        if type(data) is type(u''):
            # data maybe unicode but with another encoding specified
            data = data.encode('UTF-8')
        encoding = guess_encoding(data)
        if encoding is None:
            try:
                site_props = self.portal_properties.site_properties
                encoding = site_props.getProperty('default_charset', 'UTF-8')
            except:
                encoding = 'UTF-8'
        return encoding

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_delObjects')
    def manage_delObjects(self, ids, REQUEST=None):
        """ delete the selected mime types """
        for id in ids:
            self.unregister(self.lookup(id)[0])
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addMimeType')
    def manage_addMimeType(self, id, mimetypes, extensions, icon_path, binary=0,
                           REQUEST=None):
        """add a mime type to the tool"""
        mt = MimeTypeItem(id, mimetypes, extensions, binary, icon_path)
        self.register(mt)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_editMimeType')
    def manage_editMimeType(self, name, new_name, mimetypes, extensions, icon_path, binary=0,
                            REQUEST=None):
        """edit a mime type by name"""
        mt = self.lookup(name)[0]
        self.unregister(mt)
        mt.edit(new_name, mimetypes, extensions, icon_path, binary)
        self.register(mt)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

InitializeClass(MimeTypesRegistry)


def split(name):
    """ split a mime type in a (major / minor) 2-uple """
    try:
        major, minor = name.split('/', 1)
    except:
        raise MimeTypeException('Malformed MIME type (%s)' % name)
    return major, minor
